package org.jeecg.modules.financeManage.service;

import org.jeecg.modules.basedata.entity.FdPassFeeClassProject;
import org.jeecg.modules.financeManage.entity.FdHandoverfeeB;
import org.jeecg.modules.financeManage.entity.FdHandoverfee;
import com.baomidou.mybatisplus.extension.service.IService;
import org.jeecg.modules.projectManagement.entity.FdContract;
import org.jeecg.modules.projectManagement.entity.FdProjectFile;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;

/**
 * @Description: 财务过手费主表
 * @author： jeecg-boot
 * @date：   2019-07-13
 * @version： V1.0
 */
public interface IFdHandoverfeeService extends IService<FdHandoverfee> {

	/**
	 * 添加一对多
	 * 
	 */
	public void saveMain(FdHandoverfee fdHandoverfee, List<FdHandoverfeeB> fdHandoverfeeBList) ;
	
	/**
	 * 修改一对多
	 * 
	 */
	public void updateMain(FdHandoverfee fdHandoverfee, List<FdHandoverfeeB> fdHandoverfeeBList);

	/**
	 * 修改一对多
	 *
	 */
	public void updateAll(FdHandoverfee fdHandoverfee, List<FdHandoverfeeB> fdHandoverfeeBList);

	/**
	 * 删除一对多
	 */
	public void delMain(String id);
	
	/**
	 * 批量删除一对多
	 */
	public void delBatchMain(Collection<? extends Serializable> idList);

	/**
	 * 通过合同号，查询合同所有关联数据
	 * @param code
	 * @return
	 */
	public FdProjectFile findContractByCode(String pkProjectFile);

	/**
	 * 通过过手费编码，查询所有关联数据
	 * @param code
	 * @return
	 */
	public List<FdPassFeeClassProject> findHandoverfeeByCode(String code);

	/**
	 * 查询重复过手费号码
	 * @param code
	 * @return
	 */
	public int fingMoreCode(String code,String id);

	/**
	 * 通过合同编码，查询收款条件
	 * @param code
	 * @return
	 */
	public List<FdHandoverfeeB> selectClause(String code);

	/**
	 * 查询合同下拉
	 * @return
	 */
	public List<FdHandoverfee> selectContract();

	/**
	 * 审核
	 * @param ids
	 * @param state
	 */
	public void toExamine(String ids,String state);
}
